@@ -46,7 +46,7 @@
 #include "vdef.h"
 #include "vrt.h"
 
-#define CACHE_H_INCLUDED
+#define CACHE_H_INCLUDED	// After vrt.h include.
 
 #include "miniobj.h"
 #include "vas.h"
@@ -90,7 +90,6 @@ enum {
 struct VSC_lck;
 struct VSC_main;
 struct VSC_main_wrk;
-struct backend;
 struct ban;
 struct ban_proto;
 struct cli;
@@ -104,6 +103,7 @@ struct transport;
 struct worker;
 struct listen_sock;
 struct vcf;
+struct req_step;
 
 #define DIGEST_LEN		32
 
@@ -116,30 +116,18 @@ typedef struct {
 
 /*--------------------------------------------------------------------*/
 
-enum req_step {
-	R_STP_NONE = 0,
-#define REQ_STEP(l, u, arg)	R_STP_##u,
-#include "tbl/steps.h"
-};
-
-enum fetch_step {
-	F_STP_NONE = 0,
-#define FETCH_STEP(l, U, arg)	F_STP_##U,
-#include "tbl/steps.h"
-};
-
-/*--------------------------------------------------------------------*/
-
 struct lock { void *priv; };	// Opaque
 
 /*--------------------------------------------------------------------
  * Workspace structure for quick memory allocation.
  */
 
+#define WS_ID_SIZE 4
+
 struct ws {
 	unsigned		magic;
 #define WS_MAGIC		0x35fac554
-	char			id[4];		/* identity */
+	char			id[WS_ID_SIZE];	/* identity */
 	char			*s;		/* (S)tart of buffer */
 	char			*f;		/* (F)ree/front pointer */
 	char			*r;		/* (R)eserved length */
@@ -232,6 +220,7 @@ enum task_prio {
 	TASK_QUEUE__END
 };
 
+#define TASK_QUEUE_HIGHEST_PRIORITY TASK_QUEUE_BO
 #define TASK_QUEUE_CLIENT(prio) \
 	(prio == TASK_QUEUE_REQ || prio == TASK_QUEUE_STR)
 
@@ -247,7 +236,7 @@ struct worker {
 	struct VSC_main_wrk	*stats;
 	struct vsl_log		*vsl;		// borrowed from req/bo
 
-	struct pool_task	task;
+	struct pool_task	task[1];
 
 	vtim_real		lastused;
 	int			strangelove;
@@ -394,7 +383,7 @@ struct busyobj {
 	struct worker		*wrk;
 
 	struct vfp_ctx		*vfc;
-	const char		*filter_list;
+	const char		*vfp_filter_list;
 
 	struct ws		ws[1];
 	uintptr_t		ws_bo;
@@ -409,9 +398,9 @@ struct busyobj {
 
 	struct http_conn	*htc;
 
-	struct pool_task	fetch_task;
+	struct pool_task	fetch_task[1];
 
-#define BO_FLAG(l, r, w, d) unsigned	l:1;
+#define BO_FLAG(l, r, rr, rw, f, d) unsigned	l:1;
 #include "tbl/bo_flags.h"
 
 	/* Timeouts */
@@ -439,6 +428,8 @@ struct busyobj {
 
 	uint16_t		err_code;
 	const char		*err_reason;
+
+	const char		*client_identity;
 };
 
 
@@ -454,13 +445,19 @@ struct reqtop {
 
 struct req {
 	unsigned		magic;
-#define REQ_MAGIC		0x2751aaa1
+#define REQ_MAGIC		0xfb4abf6d
 
-	enum req_step		req_step;
 	body_status_t		req_body_status;
 	enum sess_close		doclose;
 	unsigned		restarts;
 	unsigned		esi_level;
+
+	/* Delivery mode */
+	unsigned		res_mode;
+#define RES_ESI			(1<<4)
+#define RES_PIPE		(1<<7)
+
+	const struct req_step	*req_step;
 	struct reqtop		*top;	/* esi_level == 0 request */
 
 #define REQ_FLAG(l, r, w, d) unsigned	l:1;
@@ -471,7 +468,7 @@ struct req {
 
 	struct sess		*sp;
 	struct worker		*wrk;
-	struct pool_task	task;
+	struct pool_task	task[1];
 
 	const struct transport	*transport;
 	void			*transport_priv;
@@ -525,11 +522,6 @@ struct req {
 	struct vdp_ctx		*vdc;
 	const char		*filter_list;
 
-	/* Delivery mode */
-	unsigned		res_mode;
-#define RES_ESI			(1<<4)
-#define RES_PIPE		(1<<7)
-
 	/* Transaction VSL buffer */
 	struct vsl_log		vsl[1];
 
@@ -600,7 +592,7 @@ void BAN_Abandon(struct ban_proto *b);
 
 /* cache_cli.c [CLI] */
 extern pthread_t cli_thread;
-#define ASSERT_CLI() do {assert(pthread_self() == cli_thread);} while (0)
+#define ASSERT_CLI() do {assert(pthread_equal(pthread_self(), cli_thread));} while (0)
 
 /* cache_http.c */
 unsigned HTTP_estimate(unsigned nhttp);
@@ -648,12 +640,14 @@ int HTTP_IterHdrPack(struct worker *, struct objcore *, const char **);
 #define HTTP_FOREACH_PACK(wrk, oc, ptr) \
 	 for ((ptr) = NULL; HTTP_IterHdrPack(wrk, oc, &(ptr));)
 const char *HTTP_GetHdrPack(struct worker *, struct objcore *, const char *hdr);
-enum sess_close http_DoConnection(struct http *hp);
+enum sess_close http_DoConnection(struct http *hp, enum sess_close sc_close);
+int http_IsFiltered(const struct http *hp, unsigned u, unsigned how);
 
-#define HTTPH_R_PASS	(1 << 0)	/* Request (c->b) in pass mode */
-#define HTTPH_R_FETCH	(1 << 1)	/* Request (c->b) for fetch */
-#define HTTPH_A_INS	(1 << 2)	/* Response (b->o) for insert */
-#define HTTPH_A_PASS	(1 << 3)	/* Response (b->o) for pass */
+#define HTTPH_R_PASS		(1 << 0)	/* Request (c->b) in pass mode */
+#define HTTPH_R_FETCH		(1 << 1)	/* Request (c->b) for fetch */
+#define HTTPH_A_INS		(1 << 2)	/* Response (b->o) for insert */
+#define HTTPH_A_PASS		(1 << 3)	/* Response (b->o) for pass */
+#define HTTPH_C_SPECIFIC	(1 << 4)	/* Connection-specific */
 
 #define HTTPH(a, b, c) extern char b[];
 #include "tbl/http_headers.h"
@@ -706,7 +700,7 @@ const void *ObjGetAttr(struct worker *, struct objcore *, enum obj_attr,
 typedef int objiterate_f(void *priv, unsigned flush,
     const void *ptr, ssize_t len);
 #define OBJ_ITER_FLUSH	0x01
-#define OBJ_ITER_FINAL	0x02
+#define OBJ_ITER_END	0x02
 
 int ObjIterate(struct worker *, struct objcore *,
     void *priv, objiterate_f *func, int final);
@@ -717,6 +711,11 @@ int ObjGetDouble(struct worker *, struct objcore *, enum obj_attr, double *);
 int ObjGetU32(struct worker *, struct objcore *, enum obj_attr, uint32_t *);
 int ObjGetU64(struct worker *, struct objcore *, enum obj_attr, uint64_t *);
 int ObjCheckFlag(struct worker *, struct objcore *, enum obj_flags of);
+
+/* cache_req_body.c */
+ssize_t VRB_Iterate(struct worker *, struct vsl_log *, struct req *,
+    objiterate_f *func, void *priv);
+
 /* cache_session.c [SES] */
 
 #define SESS_ATTR(UP, low, typ, len)					\
@@ -778,8 +777,6 @@ void WRK_BgThread(pthread_t *thr, const char *name, bgthread_t *func,
 
 void WS_Init(struct ws *ws, const char *id, void *space, unsigned len);
 
-/* WS_Reserve(): Use WS_ReserveSize() or WS_ReserveAll() */
-unsigned WS_Reserve(struct ws *ws, unsigned bytes) v_deprecated_;
 unsigned WS_ReserveSize(struct ws *, unsigned);
 unsigned WS_ReserveAll(struct ws *);
 unsigned WS_ReserveLumps(struct ws *ws, size_t sz);
@@ -792,19 +789,39 @@ void *WS_Alloc(struct ws *ws, unsigned bytes);
 void *WS_Copy(struct ws *ws, const void *str, int len);
 uintptr_t WS_Snapshot(struct ws *ws);
 int WS_Overflowed(const struct ws *ws);
-void *WS_Printf(struct ws *ws, const char *fmt, ...) v_printflike_(2, 3);
+const char *WS_Printf(struct ws *ws, const char *fmt, ...) v_printflike_(2, 3);
 int WS_Inside(const struct ws *, const void *, const void *);
 void WS_Assert_Allocated(const struct ws *ws, const void *ptr, ssize_t len);
 
 void WS_VSB_new(struct vsb *, struct ws *);
 char *WS_VSB_finish(struct vsb *, struct ws *, size_t *);
 
+/* REL_20210315 remove */
 static inline char*
 WS_Front(const struct ws *ws)
 {
+	AN(ws->r);
 	return ws->f;
 }
 
+static inline void *
+WS_Reservation(const struct ws *ws)
+{
+
+	WS_Assert(ws);
+	AN(ws->r);
+	AN(ws->f);
+	return (ws->f);
+}
+
+static inline unsigned
+WS_ReservationSize(const struct ws *ws)
+{
+
+	AN(ws->r);
+	return (ws->r - ws->f);
+}
+
 /* cache_rfc2616.c */
 void RFC2616_Ttl(struct busyobj *, vtim_real now, vtim_real *t_origin,
     float *ttl, float *grace, float *keep);
@@ -813,20 +830,6 @@ int RFC2616_Do_Cond(const struct req *sp);
 void RFC2616_Weaken_Etag(struct http *hp);
 void RFC2616_Vary_AE(struct http *hp);
 
-/*
- * A normal pointer difference is signed, but we never want a negative value
- * so this little tool will make sure we don't get that.
- */
-
-static inline unsigned
-pdiff(const void *b, const void *e)
-{
-
-	assert(b <= e);
-	return
-	    ((unsigned)((const unsigned char *)e - (const unsigned char *)b));
-}
-
 #define Tcheck(t) do {						\
 		AN((t).b);					\
 		AN((t).e);					\
@@ -851,10 +854,12 @@ Tlen(const txt t)
  */
 #define W_TIM_real(w) ((w)->lastused = VTIM_real())
 
-#define PAN_CheckMagic(vsb, ptr, exp)					\
-	do {								\
-		if ((ptr)->magic != (exp))				\
-			VSB_printf((vsb),				\
-			    "MAGIC at %p is 0x%08x (Should be: %s/0x%08x)\n", \
-			    ptr, (ptr)->magic, #exp, exp);		\
-	} while(0)
+int PAN_dump_struct2(struct vsb *vsb, int block, const void *ptr,
+    const char *smagic, unsigned magic, const char *fmt, ...)
+    v_printflike_(6,7);
+
+#define PAN_dump_struct(vsb, ptr, magic, ...)		\
+    PAN_dump_struct2(vsb, 1, ptr, #magic, magic, __VA_ARGS__)
+
+#define PAN_dump_oneline(vsb, ptr, magic, ...)		\
+    PAN_dump_struct2(vsb, 0, ptr, #magic, magic, __VA_ARGS__)
